;-----------------------------------------------------------------------------
;NAME
;   abbrev.ml -- front end for local-abbreviation facility
;
;SYNOPSIS
;   abbreviate-word	-- interactively set up a local word abbreviation
;
;KEY BINDINGS
;   None.
;
;DESCRIPTION
;   This fairly trivial function picks up the word containing the cursor and
;queries the user for an expansion. The word is then added to the local abbrev
;table with that expansion.
;
;HISTORY
; {2}	17-Sep-84  David Medinets (dm) at unipress
;	added  an  error-occured so  that if  you are  at the  end of the
;	buffer the function will not bomb.  I  also switched  the args to
;	define-local-abbrev to agree with the header.
;
; {1}  3-Jul-84  Eric S. Raymond at Rabbit Software
;	Added this documentation header.
;-----------------------------------------------------------------------------

(defun
   (abbreviate-word w
      (save-excursion
	 (error-occurred (forward-character))
	 (provide-prefix-argument prefix-argument (backward-word))
	 (set-mark)
	 (provide-prefix-argument prefix-argument (forward-word))
	 (setq w (region-to-string)))
         (define-local-abbrev
	    w (get-tty-string (concat ": abbreviate-word \"" w "\" by: ")))
	 (novalue)
   )
)

; abbrev.ml ends here --------------------------------------------------------
