;---------------------------------------------------------------------------
;NAME
; capword.ml
;
;SYNOPSIS
; 
; case-flip-character	-- flip case of the character following dot
; 
; upper-case-word - this function will upper casify every letter in the word
; 		    in which to dot is located
; lower-case-word - this lower casifies every letter in the word in which the
; 		    dot is currently located
; capitalize-word - this will capitalize the first letter in the word in
;		    which the dot is currently located.  All other characters
; 		    in the word will be lower case.
;
;KEYBINDINGS
; 
;        keys            new functions           old functions
; 	ESC-U		upper-case-word		case-word-upper
; 	ESC-L		lower-case-word		case-word-lower
; 	ESC-C		capitalize-word		case-word-capitalize
;       \^^ is recommended for case-flip-character.
;DESCRIPTION
;
; This package provides reasonable functions to mess with the case of The
; current word. All of them operate on the word that contains or is immediately
; after the cursor, and move the cursor to the next word when they are
; done. Normally they are bound to ESC-U, ESC-L, and ESC-C.
; 
;SEE ALSO
;
;FILES
;
;BUGS
;
;HISTORY
 ;  {2}	19-Oct-84  David Medinets (dm) at unipress
 ;	Moved the defun for case-file-character from case-flip.ml to here
 ;	and deleted that file.  
 ;
; {1}	18-Sep-84  David Medinets (dm) at unipress
;	hopefully   made   the   function   more   efficient   and   less
;	space-consuming by making them all  call &&capword-internal which
;	does  call  the hard  work.  The macro &&capword-macro is changed
;	depending on what work is to be done.  
;
; Brian Reid 1982
; 

(declare-global &&capword-cmd)

(defun
    (&&capword-internal org-dot
        (while (& (looking-at "\\w") (! (bobp)))
           (backward-character))
        (setq org-dot (- (dot) 1))
	(if (bobp) 
           (forward-word)
           (backward-word)
	)
        (if (< (- (dot) 1) org-dot)
           (progn
             (forward-word)
             (forward-word)
           )
        )
    )

    (upper-case-word
        (&&capword-internal)
        (case-word-upper)
    )

    (lower-case-word
        (&&capword-internal)
        (case-word-lower)
    )

    (capitalize-word
       (&&capword-internal)
       (case-word-capitalize)
    )

    (case-flip-character c
       (prefix-argument-loop
    	   (if (! (eobp))
	      (progn (setq c (following-char))
		     (delete-next-character)
		     (insert-character
			(if (& (<= 'a' c) (<= c 'z'))
                           (- c 040)
			    (& (<= 'A' c) (<= c 'Z'))  
                           (+ c 040)
			   c
			)
		     )
	      )
	   )
       )
    (novalue)
    )
)

(global-rebind "upper-case-word" "case-word-upper"      "\eu")
(global-rebind "lower-case-word" "case-word-lower"      "\el")
(global-rebind "capitalize-word" "case-word-capitalize" "\el")

(bind-to-key "upper-case-word" "\eu")
(bind-to-key "lower-case-word" "\el")
(bind-to-key "capitalize-word" "\ec")

(novalue)
