;---------------------------------------------------------------------------
;NAME    comments.ml
;
;SYNOPSIS
; box-region - box in C comments
;
;DESCRIPTION
; box-region -  encloses the region between dot and mark
;		in a box comment
; 
;			/*********************\
;			* 		      *
;			* this is an example  *
;			* of a boxed region   *
;			* 		      *
;			\*********************/
;
; debox-region - removes box comments as stated above.
;
; edit-block-comment - deboxes the region narrows the region and does
; 			a recursive edit on the comment.  justify-paragraph
; 			is bound to escape-j but escape-j is rebound upon
; 			returning.
;
;HISTORY
;---------------------------------------------------------------------------
;

(defun
    (box-region longest len
	   (save-excursion 
	       (narrow-region)
	       (beginning-of-file)
	       (while (& (eolp) (! (eobp)))
		      (delete-next-character))
	       (insert-string "\n")
	       (setq longest 0)
	       (error-occurred 
		   (while (! (eobp))
			  (end-of-line)
			  (if (> (setq len (current-column)) longest)
			      (setq longest len))
			  (forward-character)))
	       (end-of-file)
	       (while (& (! (bobp)) (= (preceding-char) '\n'))
		      (delete-previous-character))
	       (insert-string "\n\n")
	       (beginning-of-file)
	       (insert-string "/*")
	       (setq len 0)
	       (while (< len longest)
		      (insert-string "*")
		      (++ len))
;		      (setq len (+ len 1)))
	       (insert-string "*\\\n")
	       (error-occurred
		   (while (! (eobp))
			  (insert-string "* ")
			  (end-of-line)
			  (to-col (+ longest 3))
			  (insert-string " *")
			  (forward-character)))
	       (end-of-file)
	       (insert-string "\\*")
	       (setq len 0)
	       (while (< len longest)
		      (insert-string "*")
		      (++ len) )
;		      (setq len (+ len 1)))
	       (insert-string "*/\n")
	       (widen-region)
	   )))

(defun
    (debox-region
	   (save-excursion 
	       (save-restriction 
		   (narrow-region)
		   (beginning-of-file)
		   (error-occurred (re-replace-string "/\\*[\\*]*" ""))
		   (if (eolp) (delete-next-character))
		   (error-occurred (re-replace-string "[\\*]*\\*/[\n]*" ""))
		   (error-occurred (re-replace-string "^\\* " ""))
		   (error-occurred (re-replace-string "[ \t]*\\*$" ""))))))

(defun (edit-block-comment
	   (save-window-excursion esc-j mode
	       (end-of-line)
	       (re-search-reverse "^/\\*")
	       (set-mark)
	       (forward-character)(forward-character)
	       (search-forward "*/")
	       (if (eolp) (forward-character))
	       (debox-region)
	       (setq esc-j (local-binding-of "\ej"))
	       (local-bind-to-key "justify-paragraph" "\ej")
	       (setq left-margin 0)
	       (setq right-margin 75)
	       (setq prefix-string "")
	       (narrow-region)
	       (message "Type ^C to resume editing")
	       (setq mode mode-string)
	       (setq mode-string "Comment edit")
	       (save-excursion 
	       (recursive-edit))
	       (setq mode-string mode)
	       (local-bind-to-key esc-j "\ej")
	       (beginning-of-file)
	       (set-mark)
	       (end-of-file)
	       (widen-region)
	       (box-region)
	   )
       )
)


