;---------------------------------------------------------------------------
;NAME	common.ml
;
;SYNOPSIS
;   Functions that are useful everywhere.
;
;	confirm				prompts for yes/no answer in 
;					minibuffer, returns non-zero
;					for "yes" answer, zero for "no" answer
;	buffer-not-modified		sets buffer-is-modified to 0.
;	copy-region-to-kill-buffer	equivalent to killing and restoring
;					the region without modifying the
;					buffer.
;	revert-file			Deletes the current buffer without
;					writing the file, reads the file
;					fresh from disk.
;	name-keyboard-macro		Assigns a name to the currently
;					defined keyboard macro.
;	mark-whole-buffer		puts mark at end of file,
;					dot at beginning of file.
;       pop-out-of-emacs		writes-modified-files and pauses.
;       current-date			returns a string of DD-MMM-YY
;
;KEYBINDINGS
;
;DESCRIPTION
;
;SEE ALSO
;
;BUGS
;
;HISTORY
;
;----------------------------------------------------------------------


(defun
  (confirm
    (c= (string-to-char (substr (get-tty-string (arg 1)) 1 1)) 'y')
  )
)

(defun (buffer-not-modified (setq buffer-is-modified 0)))

(defun (copy-region-to-kill-buffer (copy-region-to-buffer "Kill buffer")))

(defun (revert-file name
	(setq name (current-file-name))
	(delete-buffer (current-buffer-name))
	(visit-file name)
	(novalue)))

(defun (name-keyboard-macro
	(define-keyboard-macro (arg 1 "Name: "))
	(novalue)))

(defun
  (mark-whole-buffer
    (end-of-file)
    (set-mark)
    (beginning-of-file)
    (novalue)
  )
)

(defun 
  (pop-out-of-emacs
    (write-modified-files)
    (pause-emacs)
  )
)

(defun 
  (current-date time-string str
    (setq time-string (current-time))
    (setq str (concat (substr time-string 9 2)  "-"
                      (substr time-string 5 3)  "-"
                      (substr time-string -2 2)
              )
    ) 
    str
  )
)
