;-----------------------------------------------------------------------------
;NAME	compile.ml -- enhanced compile-it command
;
;SYNOPSIS
;   new-compile-it	-- replaces compile-it
;
;KEY BINDINGS
;   new-compile-it	compile-it	^X^E
;
;DESCRIPTION
;	This package  provides a  replacement for  the standard ^X^E/^X^N
;	compilation package.  It operates pretty much the same as the old
;	except   that   the   process   control   facilities   are  used.
;	Consequently, while a compilation is going on you can  go off and
;	do  other  things  and  (a  major  win)  you  can  interrupt  the
;	compilation partway through.  
;
;HISTORY
 ; {3}	28-Feb-85  David Medinets (dm) at unipress
 ;	Added  a  hook  called  before-newcompile-hook.    This  was done
 ;	because I need to do certain things before each compile.  To take
 ;	advantage  of  this  feature  define  a  function   in  either  a
 ;	.emacs_init or .emacs_pro called before-newcompile-hook.
 ;
 ; {2}	18-Feb-85  David Medinets (dm) at unipress
 ;	replaced  the  literal  in  the  call   to  execute-command  with
 ;	default-error-message-format.    This  was  done   under  on  the
 ;	recommendation of ESR.
 ;
;  {1}	 1-July-84  Eric S. Raymond at Rabbit Software
;	Added rebinds, changed documentation to standard form
;-----------------------------------------------------------------------------

(autoload "execute-command" "execute.ml")

(if (! (is-bound &cmpl-last-command))
  (setq-default &cmpl-last-command "")
)

(if (! (is-bound &cmpl-buffer))
  (setq-default &cmpl-buffer "compilation")
)

(defun
  (new-compile-it ncicmd
    (if prefix-argument-provided
      (progn
	(setq ncicmd (arg 1 ": compile-it using command: "))
        (if (= ncicmd "")
          (if (= &cmpl-last-command "")
	    (error-message "No previous command")
	    (setq ncicmd &cmpl-last-command)
          )
	  (setq &cmpl-last-command ncicmd)
        )
      )
      (setq ncicmd "make -k")
    )
    (error-occurred (before-newcompile-hook))
    (write-modified-files)
    (execute-command ncicmd &cmpl-buffer default-error-message-format)
  )
)

(global-rebind "new-compile-it" "compile-it" "\^x\^e")

(novalue)