;-----------------------------------------------------------------------------
;NAME
;   describe.ml -- describe MLisp commands and variables
;
;SYNOPSIS
;   describe-command		-- describe an MLisp command
;   describe-variable		-- describe an MLisp variable
;
;KEY BINDINGS
;   None.
;
;DESCRIPTION
;   This package gives access to on-line documentation for Emacs commands
; using a special database.  Alternative definitions of these commands
; are given in the info.ml library.
;
;SEE ALSO
;   info.ml
;
;FILES
;   describe database in the directory given by the global variable
;	database-directory, normally defined in site global profile.
;
;HISTORY
;   05-July-84  Eric S. Raymond at Rabbit Software
;	Added this documentation.
;-----------------------------------------------------------------------------

(extend-database-search-list
	"describe"
	(concat database-directory "describe"))

(defun
    (describe-command word
	(setq word (get-tty-command ": describe-command "))
	(save-excursion
	    (temp-use-buffer "Help")
	    (erase-buffer)
	    (if (error-occurred (fetch-database-entry "describe" word))
		(error-message (concat "No help concerning " word))
		(progn
		    (pop-to-buffer "Help")
		    (setq buffer-is-modified 0)
		    (beginning-of-file))
	    )
	    (novalue)
	))
    
    (describe-variable word
	(setq word (get-tty-variable ": describe-variable "))
	(save-excursion
	    (temp-use-buffer "Help")
	    (erase-buffer)
	    (if (error-occurred (fetch-database-entry "describe" word))
		(error-message (concat "No help concerning " word))
		(progn
		    (pop-to-buffer "Help")
		    (setq buffer-is-modified 0)
		    (beginning-of-file))
	    )
	    (novalue)
	))
)

; describe.ml -----------------------------------------------------------------
