;---------------------------------------------------------------------------
;NAME
;desword.ml
;
;SYNOPSIS
; describe emacs functions or variables from database
;
;DESCRIPTION
;
; This package implements the describe-word-in-buffer function.
;
;HISTORY
;
;----------------------------------------------------------------------


(declare-global edit-name)

(extend-database-search-list "subr-names"
    (concat database-directory "quickinfo"))

(error-occurred
    (extend-database-search-list "subr-names"
	(concat (getenv "HOME") "/.subr-names")))

(defun
  (describe-word-in-buffer subr-name
    (if (> prefix-argument 1)
      (progn
	(error-occurred (forward-character))
	(backward-word)
	(set-mark)
	(forward-word)
	(edit-description (region-to-string))
      )
      (save-excursion
	(error-occurred (forward-character))
	(backward-word)
	(set-mark)
	(forward-word)
	(setq subr-name (region-to-string))
	(temp-use-buffer "subr-help")
	(erase-buffer)
	(if (error-occurred (fetch-database-entry "subr-names" subr-name))
	  (error-message (concat "No help for " subr-name))
	  (progn
	    (beginning-of-file)
	    (set-mark)
	    (end-of-line)
            (message (region-to-string))
          )
        )
      )
    )
    (novalue)
  )
    
  (edit-description
	(setq edit-name (arg 1 ": edit-description (of routine) "))
	(pop-to-buffer "Edit description")
	(erase-buffer)
	(if (error-occurred (fetch-database-entry "subr-names" edit-name))
	    (message "New entry."))
	(setq mode-line-format
	    (concat "   Editing database entry for "
	        edit-name "   hit ^C to update"))
	(local-bind-to-key "replace-db-entry" "\^X\^S")
	(local-bind-to-key "replace-db-entry" "\^X\^F")
	(local-bind-to-key "replace-db-entry" "\^X\^W")
	(local-bind-to-key "replace-db-entry" "\^X\^C")
	(local-bind-to-key "replace-db-entry" "\^C")		; ^C
	(novalue)
    )

    (replace-db-entry
	(put-database-entry "subr-names" edit-name)
	(delete-window)
	(novalue)
    )
)

(novalue)
