;-------------------------------------------------------------------------
;NAME
;   finger.ml -- extract a full name correctly from a finger entry
;
;SYNOPSIS
;   finger-name	-- return user's full name as a string
;
;DESCRIPTION
;   The (users-full-name) function can give confusing results if Emacs is
;configured to use the gecos field and that field is formatted as a finger
;database (i.e. with full name as the first one of several comma-separated
;fields). This often happens on BSD 4.1-derived UNIXes.
;   The (finger-name) function defined here solves this problem. It stops
;collecting characters for the user's name on a comma, if there is one. It
;also does the &-substitution defined in finger.
;
;SEE ALSO
;   finger(1)
;
;HISTORY
;   11-Jul-84  Eric S. Raymond at Rabbit Software
;	Created.
;-------------------------------------------------------------------------

(defun
    (finger-name
	i fname lname
	(setq fname (users-full-name))
	(setq lname (users-login-name))
	(setq i 1)
	(while (& (<= i (length fname)) (!= (substr fname (+ i 1) 1) ","))
	    (if (= (substr fname i 1) "&")
		(setq fname
		    (concat
			(substr fname 1 (- i 1))
			(char-to-string (- (string-to-char lname) 32))
			(substr lname 2 -1)
			(substr fname (+ i 1) (- i))
		    )
		)
	    )
	    (setq i (+ i 1))
	)
	(substr fname 1 i)
    )
)

; finger.ml ends here ----------------------------------------------------
