;---------------------------------------------------------------------------
; generate.ml
;
;SYNOPSIS
;
; (gen-on)  -- start generation
; 
; (gen-off) -- stop generation
; 
;KEYBINDINGS
;
; ^X[		gen-on
; 
; ^X]		gen-off
; 
;DESCRIPTION
;
; This package provides the user interface for the MLisp-from-keystrokes
; interface.  It prompts the user for a function name and will pop to a
; MLisp generation buffer.  When the user stops generation with (gen-off),
; the function is closed and the user is placed in electric-MLisp mode to
; polish off the function.  Be forewarned, the function gen-om (^[) will 
; leave the dot in the buffer that your were currently using and will place
; the code in the generation buffer silently.
;
; The way generation work is as such.  Everything you type will be inserted 
; into the buffer as commands.  For example if you do a ^B a 
; (backward-character) will be placed in the generation buffer.  If you
; typed foobar (insert-string "foobar") will be placed in the buffer.
; the way generation works can be likened to defining a keyboard macro
; EVERYTHING is placed in the buffer!
;
; This package requires the electric-mlisp-mode to operate correctly.
;
;HISTORY
;
; Wed Aug 03 16:04:29 1984  John McLaughlin  (mcl at unipress)
; 	Added comments on generation... August 3 1984
;----------------------------------------------------------------------

(defun
    (gen-on
	(save-excursion fname
	    (setq fname (get-tty-string "function name: "))
	    (temp-use-buffer "generate")
	    (if prefix-argument-provided
		(erase-buffer)
		(progn
		    (end-of-file)
		    (insert-string "\n")))
	    (insert-string "(defun (" fname "\n")
	    (start-generating-mlisp)))

    (gen-off
	(stop-generating-mlisp)
	(pop-to-buffer "generate")
	(end-of-file)
	(insert-string "))\n")
	(beginning-of-file)
	(electric-mlisp-mode)))

(bind-to-key "gen-on" "\^X[")
(bind-to-key "gen-off" "\^X]")
(novalue)
