;---------------------------------------------------------------------------
; goto.ml
;
;SYNOPSIS
;
; (goto-percent)   -- move dot to a given percent in the file
; 
;DESCRIPTION
;
; goto-percent
; - moves cursor past indicated percentage of the buffer.
; - percentage is taken from prefix if provided, is prompted for otherwise.
; - (goto-percent n) goes to the character closest to the
;   beginning of the buffer that is reported as n% in the
;   status line.
; - remember that the position of the first character
;   in the buffer is 1.
;
;HISTORY
; {1}	 2-Oct-84  David Medinets (dm) at unipress
;	Fixed so that (goto-percent 0)  will go  to the  beginning of the
;	file instead of the  second  character.   Also  added  a  binding
;	command.  The goto-line function is now in C so it isn't here
;	any more
;----------------------------------------------------------------------

(defun
  (goto-percent percent
     (if prefix-argument-provided
	(setq percent prefix-argument)
	(setq percent (arg 1 ": goto-percent "))
     )
     (if (= percent 0)							; {1} 
        (beginning-of-file)						; {1} 
   	(goto-character (+ (/ (* (buffer-size) percent) 100) 2))   
     )
     (novalue)
  )
)

(bind-to-key "goto-percent" "\^X\^P")					; {1}
(novalue)					       
