;---------------------------------------------------------------------------
;NAME grep.ml
;
;SYNOPSIS
;
; (grep pattern filenames)	-- interface to UNIX grep
; 
;
;DESCRIPTION
;
; This function is the interface to the grep program from within
; Emacs.  It is not very clean, but it appears to work.  It prompts
; for a pattern and a list of files.  It then calls up /bin/grep
; with the given arguments and /dev/null at the end.  This is necessary
; because otherwise grep won't print the filename in the message line
; if there is only one file specified.  This just forces the filename
; display.
;
;IMPLICIT INPUTS
; use-async-processes	If TRUE, runs grep as asynchronous process,
;			otherwise uses filter-region.  Must be FALSE (0)
;			on systems without asynchronous process capability.
;
;HISTORY
;  {2}  12-Jan-85  Mike Gallaher (mg) at unipres
;	modified to use the execute-command function defined in
;	execute.ml.
;
;  {1}	17-Feb-84  Stephen J. Friedl (friedl) at Kent State University
; 	created.
;----------------------------------------------------------------------

(progn m1 m2
   (setq m1 "grep.ml needs a file called ")
   (setq m2 " and it does not exist")
   (if (! (file-exists "/bin/grep"))
      (error-message (concat m1 "/bin/grep" m2)))
   (if (error-occurred (path-of "execute.ml"))
      (error-message (concat m1 "execute.ml" m2)))
)
(autoload "execute-command" "execute.ml")

(if (! (is-bound  &grep-buffer))
    (setq-default &grep-buffer	"grep output"))

(defun
    (grep pattern files cmd
       (save-excursion
          (setq pattern (arg 1 ": grep for pattern "))
          (if (= (length pattern) 0)
             (error-message "missing pattern to grep"))
             ; check for switch, if there remove the dash
             ; because we already send a switch to grep
          (if (= (substr pattern 1 1) '-')                      ; {2}
             (setq pattern (substr pattern 2 (length pattern))) ; {2}
             (setq pattern (concat " " pattern))
	  )
          (setq files (arg 2 ": files "))
	  (setq files (concat files " /dev/null"))              ; {2}
          (setq cmd (concat "/bin/grep -n" pattern " " files))

	  (execute-command cmd &grep-buffer "%f:%l")
       )
       (novalue)
    ))

