;---------------------------------------------------------------------------
;NAME
; keymap.ml
;
;SYNOPSIS
; copy-keymap - copy a given keymap
; create-keymap - make a new keymap with the same bindings as the current map
;
;DESCRIPTION
;
; copy-keymap takes 4 arguments.  The first is the prefix character like a 
; 	      ^X or ESC.  The second is the character at which to start
; 	      copying from.  For example if the first argument (the prefix
;	      character) was a ^X and the second argument (the starting
;	      character) was a 33 then copy-keymap will copy the bindings of
; 	      ^X! through ^X^?.  The third argument is the name of the keymap
; 	      that the bindings will be copied into.  The last argument is
; 	      the name of the keymap that is being copied from.  If the last
;	      argument is missing it will default to the current global
;	      keymap.
; create-keymap takes one argument.  This argument is the name of an
;		undefined keymap.  Create-keymap defines the keymap and
;		copies the current global map into the new keymap.
;HISTORY
;
; Aug 16 01:25:49 1984  John McLaughlin  (mcl at unipress)
; created.
;----------------------------------------------------------------------


(defun
    (copy-keymap &okm
	  (setq &okm (current-global-keymap))
          (remove-all-local-bindings)
	  (progn &ks &ch &pre &km &fn &okm 
; this progn generates all possible keystokes
		 (setq &pre (arg 1 "Prefix char: "  )) 
; get the prefix keystroke
		 (setq &ks (arg 2 "Starting Char: "))
		 (setq &km (arg 3 "New Keymap: "))
		 (if (> (nargs) 3)
		   (setq &okm (arg 4))
		   (setq &okm (current-global-keymap))
		 )
		 (while (< &ks 128); &ks is the current keystroke
			(setq &ch (concat &pre (char-to-string &ks))) ;
			; concatenate &pre and &ks to genrate a
			; keystroke sequence
			(use-global-map &okm)
			(setq &fn (global-binding-of &ch)) ; get binding
			(use-global-map &km)
			(error-occurred (bind-to-key &fn &ch))
			(if (& (= (substr &fn (- (length &fn) 5) 6) "prefix")
			       (!= (substr &fn 1 8) "argument")) 
                            (progn *t1* *t2* *t3*
	                        (setq *t1* &ch)
        	                (setq *t2* &km)
				(setq *t3* &okm)
				(copy-keymap *t1* 1 *t2* *t3*)
			    )
			)
			; if the current sequence is a prefix recurse
			(setq &ks (+ 1 &ks))
		 )
	  )
    )
)


(defun (create-keymap &kmn
 (setq &kmn (arg 1 "create-keymap: "))
 (define-keymap &kmn)
 (use-global-map &kmn)
 (copy-keymap "" 0 &kmn)
       )
)
