;-----------------------------------------------------------------------------
;NAME
;   lint.ml -- step through files directed lint error messages
;
;SYNOPSIS
;    parse-lint	-- parse current buffer for lint error message
;    make-lint	-- executes a "make lint" and parses the error log
;
;KEY BINDINGS
;    None.
;
;DESCRIPTION
;    The (parse-lint) function interprets the current buffer as a lint error
; log. It sets up the error list for (next-error) but doesn't otherwise change
; the environment (in particular it resets error-message-format to its
; previous value).
;    The (make-lint) function executes a lint command and parses the resulting
; error log. The command is the value of the string variable lint-production
; which can be set before the package load; it defaults to 'make lint'
; and is intended to be used with a make production like the following:
;
;lint:
;	lint -h $(CSRCS)
;
; where $(CSRCS) is the list of C files to be linted. It is recommended that
; this package be brought in by (autoload "parse-lint" "lint.ml) and
; (autoload "make-lint" "lint.ml").
;
;BUGS
;   Be prepared to wait a while...
;
;HISTORY
; {2}	18-Jan-85  Mike Gallaher (mg) at unipress
;	modified to use execute-command defined in execute.ml
;
; {1}	29-Jul-84  Eric S. Raymond (eric) at Rabbit Software
; 	Created.
;-----------------------------------------------------------------------------

(autoload "execute-command" "execute.ml")
(if (! (is-bound lint-production))
    (setq-default lint-production "make lint"))

(if (! (is-bound &lint-buffer))
    (setq-default &lint-buffer "Error-log"))

(declare-global lint-style 0)

(defun (make-lint
	(execute-command lint-production &lint-buffer "%f(%l)")
       ))

; lint.ml ends here ---------------------------------------------------------
