;--------------------------------------------------------------------------
;NAME
;   man.ml -- access to the Unix on-line documentation from Emacs
;
;SYNOPSIS
;   manual-entry	-- fetch the manual page on a given topic
;
;KEY BINDINGS
;   None.
;
;DESCRIPTION
;   This function prompts for a subject or accepts one as a first argument.
;It attempts to locate the subject via whereis(1) and pull the appropriate
;manual page into a new buffer named 'Manual Entry' with man(1).
;
;SEE ALSO
;   man(1), whereis(1), catman(1)
;
;FILES
;   /usr/man/man?/*
;   /usr/man/cat?/*
;
;HISTORY
;   04-Jul-84  Eric S. Raymond at Rabbit Software
;	Added this documentation header. The original was written by
;	James Rees at some time in the undated past.
;--------------------------------------------------------------------------

(defun 
    (manual-entry subj subjfile errmsg
	(setq subj (arg 1 "Subject: "))
	(setq errmsg "")
	(temp-use-buffer "Manual entry")
	(setq needs-checkpointing 0)
	(erase-buffer)
	(temp-use-buffer "Scratch")
	(setq needs-checkpointing 0)
	(erase-buffer)
	(set-mark)
	(filter-region
	    (concat "/usr/ucb/whereis -m -M /usr/man/cat? -f " subj))
	(previous-line)
	(if (|	(! (error-occurred (search-forward "No match."))) ; csh
		(error-occurred (search-forward " ")))		  ; sh
	    (save-excursion
		(message "No formatted entry, trying unformatted ...")
		(sit-for 0)
		(temp-use-buffer "Manual entry")
		(set-mark)
		(filter-region (concat "/usr/ucb/man " subj))
		(if (< (buffer-size) 80)
		    (progn
			(beginning-of-file)
			(set-mark)
			(end-of-line)
			(setq errmsg (region-to-string))
		    )
		)
	    )
	    (progn
		(while (! (eolp))
		    (set-mark)
		    (if (error-occurred (search-forward " "))
			(end-of-line)
			(delete-previous-character)
		    )
		    (setq subjfile (region-to-string))
		    (save-excursion
			(temp-use-buffer "Manual entry")
			(end-of-file)
			(insert-file subjfile)
		    )
		)
	    )
	)
	(if (!= (length errmsg) 0)
	    (message errmsg)
	    (progn
		(pop-to-buffer "Manual entry")
		(beginning-of-file)
		(message "Still working, be patient ...")
		(sit-for 0)
		(error-occurred (re-replace-string "_\b" ""))
		(error-occurred (re-replace-string
				   "^[A-Z][A-Z]*([0-9]*).*)$"
				   ""))
		(error-occurred (re-replace-string
				   "^Printed [0-9].*[0-9]$"
				   ""))
		(error-occurred (re-replace-string
				   "^Sun Release.*Last change:.*$"
				   ""))
		(error-occurred (re-replace-string "\n\n\n\n*" "\n\n"))
		(if (looking-at "\n\n*")
		    (progn
			(region-around-match 0)
			(erase-region)))
		(setq buffer-is-modified 0)
	    )
	)
	(novalue)
    )
)

; man.ml ends here --------------------------------------------------------
