;Copyright (c) 1984 - Community Data Processing
;sw: 7/20/84: fancy tops-20 style comment functions

(declare-global &new-comment-is-loaded
                $comment-command
)
(setq $comment-command -31415)			;for chaining

(if (! (is-bound &killring-is-loaded))		;is killring code not loaded?
    (autoload "kill-region" "killring.ml"))	;we use kill-region

; create comments consisting of comment-begin-string, comment-ID-string,
;   and comment-end-string, and puts the cursor after the ID
; if a comment already exists on the present line, put the cursor in
;   and insert an ID if there is not already one, and reposition comment
; mark is left before the ID, so you can M-/ to delete the ID
; puts comment-ID-string after comment-begin-string if there is none, but
;   then removes it if you do an immediate M-N or M-P
; also set comment column in various ways, and set ID string

(declare-buffer-specific comment-begin-string
                         comment-ID-string
                         comment-end-string
                         comment-prefix-string
                         comment-block-function
                         comment-column
                         comment-rounding
)

; setup defaults for shell scripts/makefiles (they have no suffix)

(setq-default comment-begin-string "#")		;start of a comment
(setq-default comment-ID-string " ")		;ID to put in comment
(setq-default comment-end-string "")		;end of comment
(setq-default comment-prefix-string "# ")	;start of 2nd line of comment
(setq-default comment-column 41)		;where put comment
(setq-default comment-rounding 1)		;1 extra space if past com-col

;to edit block comment w/function "foo", set the following to "foo"

(setq-default comment-block-function "")	;default to no block comments

(defun						;start of all the defuns
    
; kill comment and most surrounding white space; save on kill-ring
   (kill-comment
      (if (= (length comment-begin-string) 0)	;no begin string?
         (to-col comment-column)		;then just move to comment col
         (beginning-of-line)			;else prepare for search
      )
      (if (| (= (length comment-begin-string) 0)	;no begin string or..
	 (! (error-occurred (search-forward comment-begin-string))))
         (progn					;then have comment to kill
	    (provide-prefix-argument (length comment-begin-string)
		(backward-character))		;get to front of comment
	    (set-mark)				;and remember it
	    (if (> (length comment-end-string) 0) ;have an end string?
	       (progn				;then a little harder
		  (forward-character)		;begin & end string maybe same
		  (search-forward comment-end-string) ;pass end
		  (kill-region)		;save on kill ring
		  (delete-white-space)
		  (if (! (| (bolp) (eolp)))	;was comment in middle?
		     (insert-character ' ')
		  )
	       )                                ;then leave space
	       (progn				;simple with no end string
	          (end-of-line)			;get to end
	          (delete-white-space)	;toss flotsam
	          (kill-region)		;save on kill ring
	          (delete-white-space)
	       )
	    )
	 )
      )
    (novalue)
   )
    
; this changes the mark to be before the ID, so you can use kill-region on it
; this does not understand quoted comments, or comments inside quoted strings
; with a prefix argument (eg M-nnn M-;), does fancy things:
;   nnn=-1: do a block comment if know how
;   nnn=0: kill the present comment, saving it on kill-ring
;   nnn>0: align the comments on the next nnn lines
;   nnn<0: align the comments on the previous nnn lines
; if comment-rounding is <0, then precede comment with a TAB even if it is
;   past the comment column, else just use comment-rounding worth of spaces

   (indent-for-comment
      (setq this-command $comment-command)	;remember we were here
      (if (!= prefix-argument 1)		;are we doing something fancy?
	 (progn					;then various tests
; first check if we want a block comment and then if we have one
	   (if (& (= prefix-argument -1)(> (length comment-block-function) 0))
	      (execute-mlisp-line (concat "(" comment-block-function ")")))
	   (if (= prefix-argument 0)		;want to kill comment?
	     (kill-comment)			;then do it
	     (> prefix-argument 0)		;align following comments?
	     (provide-prefix-argument prefix-argument (down-comment-line))
             (provide-prefix-argument (- 0 prefix-argument) (up-comment-line))
	   )
	 )
         (progn					;else do normal comment
	    (end-of-line)
            (set-mark)
            (beginning-of-line)
	    (narrow-region)			;search only present line
	    (if (| (= (length comment-begin-string) 0) ;no begin string or..
		   (error-occurred (search-forward comment-begin-string)))
	       (progn				;no comment, so create
		  (if (| (! (& (bolp) (eolp)));have something on line or...
			 (= (length comment-begin-string) 0)) ;...no begin?
		     (progn			;then move to comment column
			(end-of-line)	;get to end of line
			(delete-white-space);toss junk
			(if (> (current-column) comment-column) ;past col?
			   (if (< comment-rounding 0) ;want a TAB anyway?
			      (insert-character '\t')
			      (prefix-argument-loop comment-rounding
					            (insert-character ' ')
			      )
			   )
			   (to-col comment-column) ;else real
			)
		     )
		  )
	       (setq prefix-string comment-prefix-string) ;if multi-line
	       (setq left-margin (current-column)) ;ditto
	       (insert-string comment-begin-string) ;create comment
	       (set-mark)			;so user can kill ID
	       (insert-string comment-ID-string) ;create ID
	       (insert-string comment-end-string)
	       (widen-region)		;prepare for next test
;  if at end of buffer then append newline
	       (if (eobp) (newline-and-backup))
;    stop in the middle of comment
	       (provide-prefix-argument (length comment-end-string)
		 	                 (backward-character)
	       )
	       )
	       (progn				;get in existing comment
;  get to the front of comment
		  (provide-prefix-argument (length comment-begin-string)
                                           (backward-character)
		  )
		  (if (! (bolp))		;not at front of line?
		     (progn			;then reposition comment
			(delete-white-space)	;toss old positioning
			(if (> (current-column) comment-column) ;too far?
			   (if (< comment-rounding 0) ;want a TAB anyway?
			      (insert-character '\t') ;then do it
			      (prefix-argument-loop comment-rounding
					            (insert-character ' ')
			      )
			   )
		        (to-col comment-column) ;else reposition
			)
		     )
		  )
;  get into comment
		  (provide-prefix-argument (length comment-begin-string)
                                           (forward-character)
		  )
		  (set-mark)			;remember where we start
		  (if (> (length comment-ID-string) 0) ;have an ID?
		     (progn			;then deal with it
		       (if (error-occurred (search-forward comment-ID-string))
			  (progn		;no ID, so insert one
			     (exchange-dot-and-mark) ;back to presearch
			     (set-mark)	;so user can kill ID
		             (insert-string comment-ID-string)
			  )
		       )
		     )
		  )
	       )
	    )
	 (widen-region)			;buffer back to normal
	 (novalue)
	 )
      )
   )
    
; this is used with the following two functions
; if we entered this function with dot right after the ID, and if the previous
;   command had been a comment command, then we toss the ID

(delete-comment-if-null cursor
    (setq cursor (dot))				;remember where we start
    (indent-for-comment)			;get into comment
    ;first if checks whether inner code found a non-null comment
    (if (if (> (length comment-end-string) 0)	;have an end string?
	    (if (looking-at (quote comment-end-string)) ;null comment?
		(progn				;then delete it
		    (provide-prefix-argument (length comment-end-string)
			(delete-next-character))	;delete end string
		    (provide-prefix-argument (+ (length comment-begin-string)
						(length comment-ID-string))
			(delete-previous-character));delete beginnings
		    (delete-white-space)		;toss flotsam
		    (if (! (| (bolp) (eolp)))	;was comment in middle?
			(insert-character ' '))	;then leave a space
		    0)				;say the comment was null
		1)					;else not null
	    (if (eolp)				;nothing in comment?
		(progn				;then delete it
		    (provide-prefix-argument (+ (length comment-begin-string)
						(length comment-ID-string))
			(delete-previous-character));delete beginnings
		    (delete-white-space)
		    0)				;say the comment was null
		1))					;else had a comment
	(if (& (> (length comment-ID-string) 0)	;have an ID and...
	       (= cursor (dot))			;...were sitting after it &...
	       (= (previous-command) $comment-command)) ;...prev was comment cmd?
	    (provide-prefix-argument (length comment-ID-string)
		(delete-previous-character))))	;then it was new, so toss
    (novalue))
    
(down-comment-line
    (setq this-command $comment-command)	;remember we were here
    (prefix-argument-loop
	(progn
	      (delete-comment-if-null)		;toss comment on this line?
	      (next-line)			;get on next line
	      (if (eobp)
		  (error-message "Reached end of buffer."))
	      (indent-for-comment)		;and get into comment
	      (novalue))))
    
(up-comment-line
    (setq this-command $comment-command)	;remember we were here
    (prefix-argument-loop
	(progn
	      (delete-comment-if-null)		;toss comment on this line
	      (previous-line)			;get on previous line
	      (if (bobp)
		  (error-message "Reached beginning of buffer."))
	      (indent-for-comment)		;and get into comment
	      (novalue))))
    
;setup comment-ID-string without messing with variables
(set-comment-ID-string
    (set-mark)					;remember end of ID
    (search-reverse comment-begin-string)	;get to front of comment
    (provide-prefix-argument (length comment-begin-string)
	(forward-character))			;get to front of ID
    (setq comment-ID-string (region-to-string))	;get the ID
    (exchange-dot-and-mark)			;back to the beginning
    (novalue))
    
;if prefix argument is <= 0 (eg M-0 ^X;), then remember the ID string
;   in front of the cursor
;if prefix arg is >1, then use it as the comment column; except, if prefix
;   argument is 4 (ie ^U had been typed), then align like previous comment
;otherwise use the present cursor location as the comment column

   (set-comment-column
      (if (<= prefix-argument 0)			;want to set ID?
	 (set-comment-ID-string)                 ;  then do it
	 (= prefix-argument 4)			;typed ^U prefix?
	 (progn					;then align like last comment
	    (save-excursion
	       (beginning-of-line)		;avoid our own comment
	       (if (error-occurred (search-reverse comment-begin-string))
		  (error-message "No previous comment to align to.")
	       )
	       (beginning-of-line)		;in case comment char in...
	       (search-forward comment-begin-string) ;..comment, get 1st one
	       (setq comment-column (- (current-column) 1)) ;use its column
	    )
	    (indent-for-comment)		;align our comment
	 )
	 (> prefix-argument 1)			;want to set col by number?
      (setq comment-column prefix-argument)	;then do it
      (setq comment-column (current-column))	;else set to present col
      )
   (novalue)
   )
)				;  end of defuns
