;----------------------------------------------------------------------------
;NAME
;   pwd.ml -- functions for getting and setting Emacs's current directory
;
;SYNOPSIS
;   pwd		-- return Emacs's current working directory
;   cd		-- set the current working directory
;
;DESCRIPTION
;   These function work exactly like their shell counterparts. They
;display and modify Emacs's base for relative pathname searches. The
;global variable cdpath is available to control the directories searched
;before the current working directory.
;
;HISTORY
; {2}	10-Feb-85  Mike Gallaher (mg) at unipress
;	Added  expand-file-name  to  avoid  complaint  about non-existent
;	directory when invoked with 'cd $foo<cr>'
;
; {1}	04-July-84  Eric S. Raymond (eric) at Rabbit Software
;	Added this documentation
;----------------------------------------------------------------------------

(if (! (is-bound cdpath)) (setq-default cdpath ".:~"))

(defun
   (pwd cwd
      (setq cwd (working-directory))
      (if (= (substr cwd -1 1) "/")
         (setq cwd (substr cwd 1 -1))
      )
      cwd
   )
    
   (cd dir path still-looking
      (setq dir (if (interactive) (get-tty-file ": cd ") (arg 1)))
      ; if the user typed return or null then default to the home dir.
      (if (= dir "")
         (setq dir "~")
      )
      (setq dir (expand-file-name dir))			;  {2} 
      ; if the dir is absolute or the home dir.
      (if (| (= (substr dir 1 1) "/") (= (substr dir 1 1) "~"))
         (setq still-looking (error-occurred (change-directory dir)))
	 (save-excursion 
	    (temp-use-buffer "Scratch Stuff")
	    (setq needs-checkpointing 0)
	    (erase-buffer)
	    (insert-string cdpath)
	    (beginning-of-file)
	    (while
               (progn 
		  (re-search-forward "[^:]*")
		  (region-around-match 0)
		  (setq path (region-to-string))
		  (erase-region)
		  (delete-next-character)
		  (setq still-looking
                     (error-occurred
			 (change-directory (if (= path "")
						dir
                                                (concat path "/" dir)
					   )
			 )
                     )
                  )
		  (& still-looking (! (eolp)))
               )
	       (novalue)
            )
         )
      )
      (if still-looking
	 (error-message "There is no such directory")
	 (progn 
	    (quietly-read-abbrev-file ".abbrevs")
	    (message (concat ": cd " (working-directory)))
         )
      )
      (novalue)
   )
)

; pwd.ml --------------------------------------------------------------------
