;----------------------------------------------------------------------------
;NAME
;   sentences.ml -- functions to work with sentence delimiters
;
;SYNOPSIS
;   forward-sentence	-- jump to the start of the next sentence
;   backward-sentence	-- jump to the begining of the current sentence
;
;KEY BINDINGS
;
;DESCRIPTION
;   This functions allow the user to step through a document in sentence
;units. The global sentence-delimiters may be set to define the RE search
;string used to find sentence boundaries.
;
;HISTORY
;   04-Jul-84  Eric S. Raymond at Rabbit Software
;	Added this documentation header.
;----------------------------------------------------------------------------

(if (! (is-bound sentence-delimiters))
    (progn
	(declare-global sentence-delimiters)
	(setq sentence-delimiters "[.!?][ \n\t][ \n\t]*")
    ))

(defun
   (forward-sentence
      (prefix-argument-loop 
         (if (error-occurred (re-search-forward sentence-delimiters))
            (end-of-file)
            (while (looking-at "")
               (search-forward "")
            )
         )
      )
   )

  (backward-sentence stpos searchp
     (prefix-argument-loop
        (setq stpos (dot))
        (preceding-char)
	(if (error-occurred (re-search-reverse sentence-delimiters))
	   (beginning-of-file)
        )
	(setq searchp (dot))
	(while (looking-at "")
	   (search-forward "")
        )
	(if (>= (dot) stpos)
	   (progn
	      (goto-character searchp)
              (if (error-occurred (re-search-reverse ""))
		 (beginning-of-file)
              )
              (while (looking-at "")
		 (search-forward "")
              )
           )
        )
     )
  )
)

(defun (kill-to-end-of-sentence
	(set-mark)
	(forward-sentence)
	(delete-to-killbuffer)
))
(defun (kill-to-beginning-of-sentence
	(set-mark)
	(backward-sentence)
	(delete-to-killbuffer)
))
; sentences.ml --------------------------------------------------------------
