;---------------------------------------------------------------------------
;NAME
;   srccom -- compare text in two windows
;
;SYNOPSIS
;   srccom -- source-compare between two windows
;
;KEY BINDINGS
;   None.
;
;DESCRIPTION
;   To begin the comparison, place the dot at the beginning of one of the
;two pieces of text to be compared, switch to the other window, and place
;the dot at the beginning of the other piece of text.  (If there are more
;than two windows, the two windows to be compared must be adjacent, and
;the dot must be left in the upper one.)  When this function is invoked,
;it will search forward, stopping when either a difference is encountered
;or the end of the buffer is reached.
;
;HISTORY:
;   July 4 1984 -- Eric S. Raymond at Rabbit Software cleaned up documentation
;   Dec 14 1982 -- Agin -- Completely rewritten for increased efficiency 
;   1 June 1982 -- Jerry Agin at Carnegie-Mellon University -- created
;---------------------------------------------------------------------------

(defun
    (srccom m1 size
	(setq m1 (dot))
	(setq size 1)
	(while (& (! (eobp)) (compare-strings size))
	   (setq size (* size 2))
	)
	(if (! (eobp))
	   (while (> size 0)
	      (compare-strings size)
	      (setq size (/ size 2))
	   )
	)
	(if (= (dot) m1)
	   (error-message "First characters are different!")
	)
    )

    (compare-strings csize s1 s2 cm1 cm2
	(setq csize (arg 1))
	(set-mark)
	(setq cm1 (dot))
	(goto-character (+ cm1 csize))
	(setq s1 (region-to-string))
	(next-window)	       
	(set-mark)
	(setq cm2 (dot))
	(goto-character (+ cm2 csize))
	(setq s2 (region-to-string))
	(if (= s1 s2)
	    (progn (previous-window) 1)
	    (progn
		  (goto-character cm2)
		  (previous-window)
		  (goto-character cm1)
		  0
	    )
	)
    )
)

; srccom.c ends here -------------------------------------------------------
