;----------------------------------------------------------------------------
;NAME
;   time.ml -- a clock for the mode line
;
;SYNOPSIS
;   time	-- toggle the mode-line clock on and off
;
;KEY BINDINGS
;   None.
;
;DESCRIPTION
;   This package uses the subprocess facility to set up a continuously-
;updating clock on the mode line. It will also notify you when you have
;mail.
;    The loadst-options variable passes the interval to check time at and
;optional flags. By default it is "60", specifying a time and mail check
;once per minute. Add "-s" to it to see load-average, CPU-time and disk-usage
;statistics.
; 
;SEE ALSO
;    loadst.c	-- Emacs helper for displaying time and statistics.
;
;HISTORY
;   20-July-84  Eric S. Raymond at Rabbit Software
;	Added loadst-options variable
;   05-July-84  Eric S. Raymond at Rabbit Software
;	Added documentation, changed time to act as a toggle.
;----------------------------------------------------------------------------

(declare-global time-running)
(setq time-running 0)
(if (! (is-bound loadst-options))
    (setq-default loadst-options "60"))

(defun
    (time dead
	(if (|
		(setq dead (< (process-status "clock") 0))
		(! time-running)
	    )
	    (save-excursion			; It's dead, (re)start it
		(if (! dead)
		    (kill-process "clock"))
		(setq global-mode-string "time and load")
		(setq time-running 0)
		(save-excursion 
		    (temp-use-buffer "clock")
		    (erase-buffer)
		    (setq needs-checkpointing 0)
		    (start-filtered-process 
			(concat
			    emacs-helper-bin-dir "loadst -n "
			    loadst-options
			)
			"clock" "clock-filter")
		)
	   )
	   (if (= (process-status "clock") 1)	; it's alive, kill it
		(progn 
			(kill-process "clock")
			(setq global-mode-string "")
			(message """time"" killed")
		)
		(error-message """time"" already dead")
	    )
	)		; end if
    )			; end time
)			; end defun

(defun
    (clock-filter
	(setq global-mode-string (process-output))
	(setq time-running 1)
	(sit-for 0)
    )
)

; time.ml -------------------------------------------------------------------
