;HISTORY
; {2}	17-Sep-84  David Medinets (dm) at unipress
;	There  are only  3 differences between this file and the
;	standard one:
;	
;		1> default value for &Default-Transpose-Magic is changed to 1.
;		2> a block of code has been added to transpose-word.
;		3> new-transpose-char has been written.
;
; {1}	08-Aug-84  Scott Weikart (sw) at cdp
;	Make transpose  word  work on  surrounding words  unless at eol.
;	make transpose character work on surrounding chars unless at
;	eol.
;
(declare-global &Default-Transpose-Direction
                &Default-Transpose-Follow
                &Default-Transpose-Magic
                &Column-To-Be-At)
(setq &Column-To-Be-At             1)

;        What the Global Transpose variables mean:
(setq &Default-Transpose-Direction 1)
;    Tells Transpose-line which other line to transpose with the current
;	one.  If this is set to 1 (actually your favorite non-zero number
;	will do) then Transpose-line will use the line above the current one
;	and if it is 0 Transpose-line will use the line below the current
;	one.
;
(setq &Default-Transpose-Follow 0)
;    If this is set Non-zero it will cause Transpose-line to leave
;	the cursor(dot) on the line that got transposed, and if this is
;	set to Zero it will stay at the same place in the file.
; 
(setq &Default-Transpose-Magic 1)
;	This  variable  controls   some  magic  inside   the  Transpose  Line
;	function.    If  it  is  set to  zero, Transpose-line  will behave as
;	controlled by the settings of the  above variables.   If  this is set
;	Non-Zero then  the magic  is controlled  by the  cursor position when
;	Transpose-line is invoked.   If the  cursor(dot) is  somewhere in the
;	middle of a line, then it behaves as if this variable were 0.  If the
;	cursor is at the end of a line, or  at the  beginning of  a line, the
;	magic will happen.  If  the cursor  is at  the beginning  of the line
;	Transpose-line will override the  above variable  settings and assert
;	that you want to transpose with the above line  and that  you want to
;	follow the line you were on.  If the cursor is at  the end  of a line
;	Transpose-line will assume that you want  to transpose  with the next
;	line and that you  want to  follow the  line you  were on.   The main
;	reason for this magic is so that you  can blip  lines up  and down in
;	your buffer real easily.  
;
(defun
   (transpose-line foo transpose-up follow EMS mark-pos
	;     ^- Zero - down, NonZero - up
      (setq EMS "")
      (setq follow &Default-Transpose-Follow)
      (setq transpose-up &Default-Transpose-Direction)
      (if &Default-Transpose-Magic
	 (if (bolp)
	    (progn (setq transpose-up 1) (setq follow 1))
             (eolp)
	    (progn (setq transpose-up 0) (setq follow 1))
	 )
      )
      (save-excursion
         (setq &Column-To-Be-At (current-column))
	 (beginning-of-line)
	 (if (& (bobp) transpose-up)
	    (setq EMS "Error - Can't transpose up here.")
	    (progn (set-mark)
                   (next-line)(beginning-of-line)
		   (if (! (bolp))
                      (progn (message "foo") (sit-for 5)))
		   (if (& (eobp) (! transpose-up))
		      (setq EMS "Error - Can't transpose down here.")
		      (delete-region-to-buffer "%%transpose")
		   )
	    )
	 )
      )
      (if (!= EMS "") (error-message EMS))
      (if transpose-up (previous-line) (next-line))
      (beginning-of-line)
      (yank-buffer "%%transpose")
      (if (& transpose-up follow)
	 (previous-line)
; if transpose-up and not follow - condition
	  transpose-up
	 (progn (goto-column)
		(if (& &Default-Transpose-Magic (eolp))
		   (backward-character))
	 )
; if not transpose-up and follow - condition
	  follow
	 (backward-character)
; if not transpose-up and not follow
	 (progn	(previous-line)
		(previous-line)
		(goto-column)
		(if (& &Default-Transpose-Magic (eolp))
                   (backward-character))          
	 )
      )
   )    
    
   (transpose-word CurrentWord punc EMS word-flag
      (setq EMS "")
      (save-excursion
; if we are at the end-of-line then backward-word          ; {2}
	 (if (eolp)                                        ; {1}
	    (backward-word))				   ; {1}
	 (backward-character)		                   ; {1}
; if we are now on a word char then forward-char           ; {2}
	 (if (looking-at "\\w")	                           ; {1}
	    (forward-character))                           ; {1}
; if we are in the middle of a word                        ; {1}
; then goto the front of it else to the front of the next  ; {2}
	 (if (looking-at "\\w")                            ; {1}
	    (backward-word)                                ; {1}
	    (progn                                         ; {1}
	       (error-occurred (re-search-forward "\\w"))  ; {1}
	       (backward-character)                        ; {1}
	    )						   ; {1}
	 )						   ; {1}
	 (if (bobp)					   
	    (setq EMS "Can't Transpose here!")
	    (progn (set-mark)
                   (forward-word)
		   (setq CurrentWord (region-to-string))
		   (backward-word)(backward-word)(forward-word)
		   (setq punc (region-to-string))(delete-next-word)
		   (backward-word)
		   (insert-string (concat CurrentWord punc))
	    )
	 )
      )
      (if (!= "" EMS) (error-message EMS))
   )
    
   (goto-column
      (beginning-of-line)
      (while (& (! (eolp)) (< (current-column) &Column-To-Be-At))
         (forward-character)
      )
   )
; 
; This is a emulation of the Tops-20 version of transpose-chars
; This was written uder the auspices of edit tag {1}.
; 
   (new-transpose-characters char
      (if (eolp)			;at end of line?
	 (backward-character))		;then work like old one
      (setq char (following-char))	;get the "following" char
      (delete-next-character)		;then delete it
      (backward-character)		;back over "previous" char
      (insert-character char)		;insert "following" char
      (forward-character)		;cross over old "previous"
   )
)

(save-window-excursion
    (temp-use-buffer "%%transpose")
    (setq needs-checkpointing 0)
)

(novalue)
