;---------------------------- CUT HERE ---------------------------------------
;This is the Emacs WordStar emulator which I received from Andrew
;Klossner (orca!andrew) for Gosling/Unipress Emacs. There is a BIG
;problem with it, though: most versions of Emacs will NOT let you rebind
;^X, because that's used as a prefix key. For that reason, ^J is used
;for next-line.  Note, however, that some automatic modes (C-mode,
;mlisp-mode) will (locally) rebind ^J to "newline-and-indent".  Just be
;careful, that's all. For your convenience, ^X-^J is bound to next-line.
;This is for those times when you hit ^X and really meant ^J: just hit ^J.
;
;There are some other problems: The string searching/replacing is not
;complete (it lacks the "options"), among other things. Also, the
;definition of a "word" for ^A, ^F, and ^T is a little cockeyed. I'm
;working on that one. Also, paragraph reformatting loses the initial
;indentation of the paragraph; another project.
;
;It is recommended that when you start using this, you perform the
;following:
;	1) Invoke Emacs with the load instruction shown below.
;	2) Enter the following keystrokes:
;		ESC-xdescribe-bindings		(plus "return")
;		ESC-xwrite-named-file		(emacs asks for file name)
;		WS-keys				 saves the bindings in
;						the file "WS-keys")
;
;Then print out WS-keys, and you'll have a handy reference for when you
;get stuck without remembering which key does what.
;
;You will not find yourself using the WordStar keys exclusively; some
;knowledge of Emacs is still tremendously helpful.
;The single file wordstar.ml contains all you need. Just put it in some
;directory and add that directory to your Epath environment variable. Then
;include
;
;	(load "wordstar.ml")
;
;in your $HOME/.emacs_pro file.
;
;There is nothing super-special about any of this. You are free to
;rebind keys however you like, with the urgent request that you not
;rebind ESC-x unless you know what you're doing. Otherwise, you may
;never make it out alive.
;
;				-- Allan Pratt
;				Jan 23 1985
;		...decvax!ihnp4!inuxc!iuvax!apratt
;
;-----------------------------------CUT HERE----------------------------------

;------------------------------ wordstar.ml ------------------------------
; This is a WordStar emulator for Gosling/Unipress Emacs. It is placed
; unsupported in the public domain (implicitly) by the original author,
; Andrew Klossner (orca!andrew), and (explicitly) by the modifier, Allan
; Pratt (iuvax!apratt). Further distribution should, out of courtesy, at
; least, contain this notice.

; Note that ^X is hard-wired in emacs to a prefix key and so cannot be
;   used for down-line.  Instead, ^J is used.

; Change the definition of a word to match that of WordStar.
; Word movement always stops with the first character of a word.
; A word is delimited by spaces and/or control characters;
;   punctuation is considered to be part of a word.

; ** Note from iuvax!apratt ** Thu Jan 26 14:01:36 1984 **
; The following is a bogus definition of a WordStar word. I am working on
;  a better one. Until then, this will have to do.

(defun (WS-forward-word
	(prefix-argument-loop
		(forward-character)
		(while (! (| (eobp)
			     (& (<= (preceding-char) 32)
				(> (following-char) 32))
			     ))
			(forward-character)
			)
		)
	(novalue)
	))
(defun (WS-backward-word
	(prefix-argument-loop
		(backward-character)
		(while (! (| (= (dot) 1)
			     (& (<= (preceding-char) 32)
				(> (following-char) 32))
			     ))
			(backward-character)
			)
		)
	(novalue)
	))
(defun (WS-delete-to-next-word
	(prefix-argument-loop
		(set-mark)
		(WS-forward-word)
		(erase-region)
		)
	(novalue)
	))

(defun (WS-newline
	   (prefix-argument-loop
	       (if (! (is-bound insert-mode-flag))
		   (newline)
		   (if (= insert-mode-flag 0)
		       (progn
			   (end-of-line)
			   (if (eobp) (newline) (forward-character))
		       )
		       (newline)
		   )
	       )
	       (novalue)
	   ))
)

; Wordstar-like searching (not completely implemented).
(declare-global WS-search-string WS-replace-string WS-replace-flag)
(setq WS-search-string "")
(setq WS-replace-string "")
(setq WS-replace-flag 2)
(defun (WS-re-search
	(if (= WS-replace-flag 0)
		(WS-search-again)
		(if (= WS-replace-flag 1)
			(WS-replace-again)
			(error-message "No search yet")
			)
		)
	(novalue)
	))
(defun (WS-search-again
	(setq V-mark (dot))
	(prefix-argument-loop
		(search-forward WS-search-string)
		)
	))
(defun (WS-search
	(setq WS-search-string (arg 1 "Search for: "))
	(setq WS-replace-flag 0)
	(WS-search-again)
	(novalue)
	))
(defun (WS-replace-again
	(setq V-mark (dot))
	(prefix-argument-loop
		(search-forward WS-search-string)
		(if (!= V-mark (dot))
			(progn
				(set-mark)
				(goto-character (- (dot)
						   (length WS-search-string)))
				(erase-region)
				(insert-string WS-replace-string)
				)
			)
		)
	))
(defun (WS-replace
	(setq WS-search-string (arg 1 "Search for: "))
	(setq WS-replace-string (arg 2 "Replace with: "))
	(setq WS-replace-flag 1)
	(WS-replace-again)
	(novalue)
	))

; Help define WordStar line-kill commands.  There are three of these:
; ^Y   -- delete entire line including newline
; ^Q^H -- delete from start of line to cursor
; ^Q^Y -- delete from cursor to end of line, not including newline
(defun (WS-kill-to-start-of-line
	(set-mark)
	(beginning-of-line)
	(erase-region)
	(novalue)
	))
(defun (WS-kill-to-end-of-line
	(set-mark)
	(end-of-line)
	(erase-region)
	(novalue)
	))
; the following was modified (written, really) by Allan Pratt, 1/18/84.
(defun (WS-kill-line
	   (prefix-argument-loop
	       (beginning-of-line)
	       (set-mark)
	       (end-of-line)
	       (if (eobp)
		   (progn (erase-region)
		       (delete-previous-character)
		   )
		   (progn (forward-character)
		       (erase-region)
		   )
	       ))
	   (novalue)
       ))

; WordStar-like buffer pointer manipulation.
; Pointers are named B K V 0 1 2 3 4 5 6 7 8 9
; ^Kx sets pointer x to dot; ^Qx moves to pointer x.
(declare-global B-mark K-mark V-mark
	m0-mark m1-mark m2-mark m3-mark m4-mark
	m5-mark m6-mark m7-mark m8-mark m9-mark)
(defun
	(goto-mark mark
		(setq mark (arg 1 "Mark: "))
		(pop-to-buffer mark)
		(goto-character mark)
	(novalue)
	))
(defun
	(WS-goto-B-mark (goto-mark B-mark))
	(WS-goto-K-mark (goto-mark K-mark))
	(WS-goto-V-mark (goto-mark V-mark))
	(WS-goto-0-mark (goto-mark m0-mark))
	(WS-goto-1-mark (goto-mark m1-mark))
	(WS-goto-2-mark (goto-mark m2-mark))
	(WS-goto-3-mark (goto-mark m3-mark))
	(WS-goto-4-mark (goto-mark m4-mark))
	(WS-goto-5-mark (goto-mark m5-mark))
	(WS-goto-6-mark (goto-mark m6-mark))
	(WS-goto-7-mark (goto-mark m7-mark))
	(WS-goto-8-mark (goto-mark m8-mark))
	(WS-goto-9-mark (goto-mark m9-mark))
	)
(defun
	(WS-set-B-mark (setq B-mark (dot)) (novalue))
	(WS-set-K-mark (setq K-mark (dot)) (novalue))
	(WS-set-V-mark (setq V-mark (dot)) (novalue))
	(WS-set-0-mark (setq m0-mark (dot)) (novalue))
	(WS-set-1-mark (setq m1-mark (dot)) (novalue))
	(WS-set-2-mark (setq m2-mark (dot)) (novalue))
	(WS-set-3-mark (setq m3-mark (dot)) (novalue))
	(WS-set-4-mark (setq m4-mark (dot)) (novalue))
	(WS-set-5-mark (setq m5-mark (dot)) (novalue))
	(WS-set-6-mark (setq m6-mark (dot)) (novalue))
	(WS-set-7-mark (setq m7-mark (dot)) (novalue))
	(WS-set-8-mark (setq m8-mark (dot)) (novalue))
	(WS-set-9-mark (setq m9-mark (dot)) (novalue))
	)

; WordStar-like block commands.
; The marked block is between mB-mark and mK-mark.
; ^K^C copies the marked block to dot.
; ^K^V moves the marked block to dot (copies it then deletes it).
; ^K^Y deletes the marked block.
; ^K^W writes the marked block to a file.
; Note that ^K^C and ^K^V move B-mark and K-mark to bound the
;   block after movement, and position V-mark to the block's old
;   position.
(defun (WS-copy-block olddot
	(setq olddot (dot))
	(save-window-excursion
		(goto-mark B-mark)
		(setq V-mark (dot))
		(set-mark)
		(goto-mark K-mark)
		(copy-region-to-buffer "block-buffer")
		)
	(goto-mark olddot)
	(prefix-argument-loop
		(setq B-mark (dot))
		(yank-buffer "block-buffer")
		(setq K-mark (dot))
		)
	(delete-buffer "block-buffer")
	(novalue)
	))
(defun (WS-move-block olddot
	(setq olddot (dot))
	(save-window-excursion
		(goto-mark B-mark)
		(setq V-mark (dot))
		(set-mark)
		(goto-mark K-mark)
		(delete-region-to-buffer "block-buffer")
		)
	(goto-mark olddot)
	(setq B-mark (dot))
	(yank-buffer "block-buffer")
	(setq K-mark (dot))
	(delete-buffer "block-buffer")
	(novalue)
	))
(defun (WS-delete-block
	(save-window-excursion
		(goto-mark B-mark)
		(set-mark)
		(goto-mark K-mark)
		(delete-region-to-buffer "block-buffer")
		)
	(delete-buffer "block-buffer")
	(novalue)
	))
(defun (WS-write-block file-name
	(setq file-name (arg 1 "File to write: "))
	(save-window-excursion
		(goto-mark B-mark)
		(set-mark)
		(goto-mark K-mark)
		(copy-region-to-buffer "block-buffer")
		(pop-to-buffer "block-buffer")
		(write-named-file file-name)
		)
	(delete-buffer "block-buffer")
	(novalue)
	))
(defun (WS-write-file-exit
	(write-current-file)
	(exit-emacs)
	))

; Wordstar-like text support: line center, left/right-margin set,
; toggle auto word fill, paragraph justify
(autoload "WS-center-line" "ws-text")
(autoload "WS-set-left-margin" "ws-text")
(autoload "WS-set-right-margin" "ws-text")
(autoload "WS-toggle-word-fill" "ws-text")
(autoload "WS-justify-paragraph" "ws-text")


; Allow multiple key-bindings in a single function call.
(defun (bind-mulkey i args fn
	(setq fn (arg 1))
	(setq args (nargs))
	(setq i 2)
	(while (<= i args)
		(bind-to-key fn (arg i))
		(setq i (+ i 1))
		)
	))

; For each keystroke beginning with ^Q, bind that keystroke also so that
; it begins with ^Xq and ^XQ.  This allows these keystrokes to be entered
; over brain-damaged network connections which don't pass ^S/^Q.
(defun (bind-mulkey-Q i args fn nextarg
	(setq fn (arg 1))
	(setq args (nargs))
	(setq i 2)
	(while (<= i args)
		(setq nextarg (arg i))
		(bind-to-key fn (concat "\^Q" nextarg))
		(bind-to-key fn (concat "\^Xq" nextarg))
		(bind-to-key fn (concat "\^XQ" nextarg))
		(setq i (+ i 1))
		)
	))

; Single keystroke bindings.
(bind-to-key "WS-backward-word"			"\^A")
(bind-to-key "WS-justify-paragraph"		"\^B")
(bind-to-key "next-page"			"\^C")
(bind-to-key "forward-character"		"\^D")
(bind-to-key "previous-line"			"\^E")
(bind-to-key "WS-forward-word"			"\^F")
(bind-to-key "delete-next-character"		"\^G")
(bind-to-key "backward-character"		"\^H")
(bind-mulkey "next-line"			"\^J" "\^X\^J" "\^Xj" "\^XJ")
(bind-to-key "WS-re-search"			"\^L")
(bind-to-key "WS-newline"			"\^M")
(bind-to-key "newline-and-backup"		"\^N")
(bind-to-key "quote-character"			"\^P")
(bind-to-key "previous-page"			"\^R")
(bind-mulkey "backward-character"		"\^S" "\^Xs" "\^XS")
(bind-to-key "WS-delete-to-next-word"		"\^T")
(bind-to-key "insert-toggle"			"\^V")
(bind-to-key "scroll-one-line-down"		"\^W")
(bind-to-key "WS-kill-line"			"\^Y")
(bind-to-key "scroll-one-line-up"		"\^Z")

; Double keystroke ^K bindings.
(bind-to-key "WS-set-0-mark"			"\^K0")
(bind-to-key "WS-set-1-mark"			"\^K1")
(bind-to-key "WS-set-2-mark"			"\^K2")
(bind-to-key "WS-set-3-mark"			"\^K3")
(bind-to-key "WS-set-4-mark"			"\^K4")
(bind-to-key "WS-set-5-mark"			"\^K5")
(bind-to-key "WS-set-6-mark"			"\^K6")
(bind-to-key "WS-set-7-mark"			"\^K7")
(bind-to-key "WS-set-8-mark"			"\^K8")
(bind-to-key "WS-set-9-mark"			"\^K9")
(bind-mulkey "WS-set-B-mark"			"\^K\^B" "\^Kb" "\^KB")
(bind-mulkey "WS-copy-block"			"\^K\^C" "\^Kc" "\^KC")
(bind-mulkey "WS-write-file-exit"		"\^K\^D" "\^Kd" "\^KD")
(bind-mulkey "WS-set-K-mark"			"\^K\^K" "\^Kk" "\^KK")
(bind-mulkey "exit-emacs"			"\^K\^Q" "\^Kq" "\^KQ")
(bind-mulkey "insert-file"			"\^K\^R" "\^Kr" "\^KR")
(bind-mulkey "write-current-file"		"\^K\^S" "\^Ks" "\^KS")
(bind-mulkey "WS-move-block"			"\^K\^V" "\^Kv" "\^KV")
(bind-mulkey "WS-write-block"			"\^K\^W" "\^Kw" "\^KW")
(bind-mulkey "WS-write-file-exit"		"\^K\^X" "\^Kx" "\^KX")
(bind-mulkey "WS-delete-block"			"\^K\^Y" "\^Ky" "\^KY")

; Double keystroke ^O bindings.
(bind-mulkey "WS-center-line"			"\^O\^C" "\^Oc" "\^OC")
(bind-mulkey "WS-set-left-margin"		"\^O\^L" "\^Ol" "\^OL")
(bind-mulkey "WS-set-right-margin"		"\^O\^R" "\^Or" "\^OR")
(bind-mulkey "WS-toggle-word-fill"		"\^O\^W" "\^Ow" "\^OW")

; Double keystroke ^Q bindings.
(bind-mulkey-Q "WS-goto-0-mark"			"0")
(bind-mulkey-Q "WS-goto-1-mark"			"1")
(bind-mulkey-Q "WS-goto-2-mark"			"2")
(bind-mulkey-Q "WS-goto-3-mark"			"3")
(bind-mulkey-Q "WS-goto-4-mark"			"4")
(bind-mulkey-Q "WS-goto-5-mark"			"5")
(bind-mulkey-Q "WS-goto-6-mark"			"6")
(bind-mulkey-Q "WS-goto-7-mark"			"7")
(bind-mulkey-Q "WS-goto-8-mark"			"8")
(bind-mulkey-Q "WS-goto-9-mark"			"9")
(bind-mulkey-Q "WS-replace" 			"\^A" "a" "A")
(bind-mulkey-Q "WS-goto-B-mark"			"\^B" "b" "B")
(bind-mulkey-Q "end-of-file"			"\^C" "c" "C")
(bind-mulkey-Q "end-of-line"			"\^D" "d" "D")
(bind-mulkey-Q "beginning-of-window"		"\^E" "e" "E")
(bind-mulkey-Q "WS-search"			"\^F" "f" "F")
(bind-mulkey-Q "WS-kill-to-start-of-line"	"\^H" "h" "H")
(bind-mulkey-Q "end-of-window"			"\^J" "j" "J")
(bind-mulkey-Q "WS-goto-K-mark"			"\^K" "k" "K")
;(bind-mulkey-Q "new-undo"			"\^P" "p" "P")
(bind-mulkey-Q "beginning-of-file"		"\^R" "r" "R")
(bind-mulkey-Q "beginning-of-line"		"\^S" "s" "S")
(bind-mulkey-Q "WS-goto-V-mark"			"\^V" "v" "V")
(bind-mulkey-Q "end-of-window"			"\^X" "x" "X")
(bind-mulkey-Q "WS-kill-to-end-of-line"		"\^Y" "y" "Y")

; TEXT functions
; Recommended key bindings:
; ^O-c   WS-center-line  (center current line)
; ^O-l   WS-set-left-margin  (prompts for and sets left margin)
; ^O-r   WS-set-right margin  (prompts for and sets right margin)
; ^O-w   WS-toggle-word-fill  (toggle auto-wrap)
; ^B     WS-justify-paragraph  (reformat current paragraph)
; ESC-^B WS-justify-paragraph-indent (reformat with extra tab indentation)

(defun (WS-center-line current-mark end-mark spaces
	(setq current-mark (dot))
	(beginning-of-line)
	(while (= ' ' (following-char))
		(delete-next-character)
		)
	(end-of-line)
	(setq end-mark (dot))
	(beginning-of-line)
	(setq spaces
		(/ (-
			(if (= right-margin 10000) 72 right-margin)
			(- end-mark (dot))
		   ) 2)
		)
	(while (> spaces 0)
		(insert-character ' ')
		(setq spaces (- spaces 1))
		)
	(goto-character current-mark)
	(novalue)
	))

(defun (WS-set-left-margin
	(setq left-margin (arg 1 "Left margin: "))
	(novalue)
	))

(defun (WS-set-right-margin
	(setq right-margin (arg 1 "Right margin: "))
	(novalue)
	))

(defun (WS-toggle-word-fill
	(if (= 10000 right-margin)
		(progn
			(setq right-margin 72)
			(setq mode-string "Fill mode")
			)
		(progn
			(setq right-margin 10000)
			(setq mode-string "Normal")
			)
		)
	(novalue)
	))

(defun (WS-justify-paragraph old-right-margin old-dot
	(if (= 10000 right-margin)
		(progn
			(setq old-right-margin right-margin)
			(setq right-margin 72)
			(justify-paragraph)
			(setq right-margin old-right-margin)
			)
		(justify-paragraph)
		)
	(message "Paragraph reformatted.")
	(novalue)
	))

(defun (WS-justify-paragraph-indent old-left-margin
	(setq old-left-margin left-margin)
	(setq left-margin (+ 8 left-margin))
	(WS-justify-paragraph)
	(setq left-margin old-left-margin)
	(novalue)
	))

; OVERWRITE MODE support -- usually toggled with ^V.

; To switch between modes, bind "insert-toggle" to a key and hit that key.
; (WordStar users might choose to bind this function to ^V.)
; Alternatively, bind "overwrite-mode" to one key and "insert-mode" to
; another, then use those two keys to switch between modes.
; The current mode is part of the highlighted mode line at the bottom
; of the window; "Normal" for insert mode, "Overwrite" for overwrite
; mode.

(setq-default insert-mode-flag 1)
(declare-buffer-specific insert-mode-flag)
(defun (self-overwrite
	(if (! (| (eobp) (eolp)))
		(if (| (!= 9 (following-char)) (= 0 (% (current-column) 8)))
			(delete-next-character)
			)
		)
	(insert-character (last-key-struck))
	))
(defun (tab-overwrite
	(if (| (eobp) (eolp))
		(insert-character 9)
		(progn
			(forward-character)
			(while (&
					(! (| (eobp) (eolp)))
					(!= 1 (% (current-column) 8))
					)
				(forward-character)
				)
			(if (!= 1 (% (current-column) 8))
				(insert-character 9)
				)
			)
		)
	))
(defun (argument-prefix-overwrite c i f1 f2
	(setq i 4)		; i is number being assembled
	(setq f1 1)		; f1 is "continue looping" flag
	(setq f2 0)		; f2 is "digit seen" flag
	(while f1
		(setq c (get-tty-character))
		(if (= c 21)
			(setq i (* i 4))
			(if (& (<= '0' c) (<= c '9'))
				(if f2
					(setq i (+ (* i 10) (- c '0')))
					(progn
						(setq i (- c '0'))
						(setq f2 1)
						)
					)
				(setq f1 0)
				)
			)
		)
	(push-back-character c)
	(return-prefix-argument i)
	))
(defun (overwrite-mode i
	(local-bind-to-key "tab-overwrite" "\^I")
	(local-bind-to-key "argument-prefix-overwrite" "\^U")
	(setq i 32)
	(while (< i 127)
		(local-bind-to-key "self-overwrite" i)
		(setq i (+ i 1))
		)
	(setq insert-mode-flag 0)
	(setq mode-string "Overwrite")
	(novalue)
	))
(defun (insert-mode
	(remove-all-local-bindings)
	(setq insert-mode-flag 1)
	(setq mode-string "Normal")
	(novalue)
	))
(defun (insert-toggle
	(if (= insert-mode-flag 0)
		(insert-mode)
		(overwrite-mode)
		)
	(novalue)
	))
(novalue)


